/*
* Copyright (C) 2011 Sunil Mohan Adapa <sunil@medhas.org>.
* Copyright (C) 2011 O S K Chaitanya <oskchaitanya@gmail.com>.
*
* Author: Sunil Mohan Adapa <sunil@medhas.org>
*         O S K Chaitanya <oskchaitanya@gmail.com>
*
* This file is part of GNOME Nonogram.
*
* GNOME Nonogram is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* GNOME Nonogram is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with GNOME Nonogram. If not, see <http://www.gnu.org/licenses/>.
*/

const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;

ClueTable = new GType(_ClueTable = {
    parent: Gtk.Table.type,
    name: "ClueTable",

    _clues: null,
    _numClueLines: null,
    _maxClueLineLength: null,

    class_init: function(klass, prototype) {
        prototype._clues = _ClueTable._clues;
        prototype._numClueLines = _ClueTable._numClueLines;
        prototype._maxClueLineLength = _ClueTable._maxClueLineLength;

        prototype.setClues = _ClueTable.setClues;
        prototype._populateClues = _ClueTable._populateClues;
    },

    init: function() {
    },

    setClues: function(clues) {
        this._clues = clues;

        // Remove existing children
        this.foreach(
            function(child) {
                child.get_parent().remove(child);
            }
        );

        // Find new dimensions
        this._numClueLines = this._clues.length;
        this._maxClueLineLength = 0;
        for (var i = 0; i < this._numClueLines; ++i) {
            if (this._maxClueLineLength < this._clues[i].length)
                this._maxClueLineLength = this._clues[i].length;
        }

        // Resize
        if (this.orientation == "vertical") {
            this.resize(this._maxClueLineLength, this._numClueLines);
        }
        else {
            this.resize(this._numClueLines, this._maxClueLineLength);
        }

        // Create new children
        this._populateClues();

        // Show the new children
        this.show_all();
    },

    _populateClues: function() {
        for (var i = 0; i < this._numClueLines; ++i) {
            var array = this._clues[i];

            for (var j = 0; j < this._maxClueLineLength; ++j) {
                var labelText = " ";
                var index = j - this._maxClueLineLength + array.length;
                if (index >= 0) {
                    labelText = array[index].count.toString();

                    var labelColor = array[index].color.toString(16);
                    labelColor = "00000000".slice(0, -labelColor.length)
                                    + labelColor;
                    labelColor = labelColor.slice(0, -2);

                    labelText = "<span color='#"
                                    + labelColor
                                    + "'><big>"
                                    + labelText
                                    + "</big></span>";
                }

                var label = new Gtk.Label({
                                              label: labelText,
                                              xalign: this.labelTextXAlign,
                                              use_markup: true
                                          });

                // Assuming orientation == "horizontal" by default
                var row = j;
                var col = i;

                if (this.orientation == "vertical") {
                    row = i;
                    col = j;
                }

                this.attach(label,
                            row, row+1, col, col+1,
                            Gtk.AttachOptions.EXPAND |
                            Gtk.AttachOptions.FILL,
                            Gtk.AttachOptions.EXPAND |
                            Gtk.AttachOptions.FILL,
                            0, 0);
            }
        }
    }
});


RowClueTable = new GType({
    parent: ClueTable.type,
    name: "RowClueTable",
    class_init: function(klass, prototype) {
        prototype.orientation = "horizontal";
        prototype.labelTextXAlign = 1.0;
    }
});


ColumnClueTable = new GType({
    parent: ClueTable.type,
    name: "ColumnClueTable",
    class_init: function(klass, prototype) {
        prototype.orientation = "vertical";
        prototype.labelTextXAlign = 0.5;
    }
});
