/*
* Copyright (C) 2011 Sunil Mohan Adapa <sunil@medhas.org>.
* Copyright (C) 2011 O S K Chaitanya <oskchaitanya@gmail.com>.
*
* Author: Sunil Mohan Adapa <sunil@medhas.org>
*         O S K Chaitanya <oskchaitanya@gmail.com>
*
* This file is part of GNOME Nonogram.
*
* GNOME Nonogram is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* GNOME Nonogram is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with GNOME Nonogram. If not, see <http://www.gnu.org/licenses/>.
*/

const Lang = imports.lang;
const Gtk = imports.gi.Gtk;
const GtkBuilder = imports.gtkbuilder;

const Config = imports.config;

MainWindow = new GType(_MainWindow = {
    parent: Gtk.Window.type,
    name: "MainWindow",

    _DASHBOARD_VIEW_PAGE: 0,
    _GAME_VIEW_PAGE: 1,

    _aboutDialog: null,
    _dashboardWidget: null,
    _gameWidget: null,
    _viewNotebook: null,

    class_init: function(klass, prototype) {
        prototype._DASHBOARD_VIEW_PAGE = _MainWindow._DASHBOARD_VIEW_PAGE;
        prototype._GAME_VIEW_PAGE = _MainWindow._GAME_VIEW_PAGE;

        prototype._aboutDialog = _MainWindow._aboutDialog;
        prototype._dashboardWidget = _MainWindow._dashboardWidget;
        prototype._gameWidget = _MainWindow._gameWidget;
        prototype._viewNotebook = _MainWindow._viewNotebook;

        prototype.connectSignalsAndChildren =
            _MainWindow.connectSignalsAndChildren;
        prototype._acquireChildren = _MainWindow._acquireChildren;
        prototype._about = _MainWindow._about;
        prototype._newGame = _MainWindow._newGame;
        prototype._startGame = _MainWindow._startGame;
        prototype._restartGame = _MainWindow._restartGame;
        prototype._switchToDashboardView = _MainWindow._switchToDashboardView;
        prototype._switchToGameView = _MainWindow._switchToGameView;
        prototype._onPuzzleActivated = _MainWindow._onPuzzleActivated;
    },

    init: function() {
    },

    connectSignalsAndChildren: function(builder) {
        var signalHandlers = {
            quit: Gtk.main_quit,
            about: Lang.bind(this, this._about),
            newGame: Lang.bind(this, this._newGame),
            restartGame: Lang.bind(this, this._restartGame),
            switchToDashboardView: Lang.bind(this,
                this._switchToDashboardView)
        };

        builder.connect_signals(signalHandlers);

        this._acquireChildren(builder);

        this._dashboardWidget.connectSignalsAndChildren(builder);
        this._gameWidget.connectSignalsAndChildren(builder);

        this._dashboardWidget.signal.puzzle_activated.connect(
            Lang.bind(this, this._onPuzzleActivated));
    },

    _acquireChildren: function(builder) {
        builder.add_from_file(Config.datadir + "/toolbar.ui");
        var toolbar = builder.get_object("toolbar");
        var mainVBox = builder.get_object("main_vbox");
        mainVBox.pack_start(toolbar, false, false, 0);

        this._viewNotebook = builder.get_object("view_notebook");

        builder.add_from_file(Config.datadir + "/dashboard-view.ui");
        this._dashboardWidget = builder.get_object("dashboardwidget");
        this._viewNotebook.append_page(this._dashboardWidget);

        builder.add_from_file(Config.datadir + "/game-view.ui");
        var gameViewVBox = builder.get_object("game_view_vbox");
        this._viewNotebook.append_page(gameViewVBox);

        this._viewNotebook.set_current_page(0);

        this._gameWidget = builder.get_object("gamewidget");

        builder.add_from_file(Config.datadir + "/about-dialog.ui");
        this._aboutDialog = builder.get_object("aboutdialog");
        this._aboutDialog.version = Config.version;
        this._aboutDialog.transient_for = this;
    },

    _about: function() {
        this._aboutDialog.run();
        this._aboutDialog.hide();
    },

    _newGame: function() {
        var fileChooserDialog = new Gtk.FileChooserDialog();
        fileChooserDialog.set_title("Open image file for puzzle");
        fileChooserDialog.add_button("Cancel", Gtk.ResponseType.CANCEL);
        fileChooserDialog.add_button("Open", Gtk.ResponseType.ACCEPT);
        fileChooserDialog.set_action(Gtk.FileChooserAction.OPEN);
        fileChooserDialog.set_transient_for(this);

        var response = fileChooserDialog.run();

        if (response == Gtk.ResponseType.ACCEPT) {
            var filename = fileChooserDialog.get_filename();
            this._startGame(filename);
        }

        fileChooserDialog.destroy();
    },

    _startGame: function(puzzlePath) {
        this._gameWidget.puzzleFile = puzzlePath;
        this._switchToGameView();
    },

    _restartGame: function() {
        this._gameWidget.restart();
    },

    _switchToGameView: function() {
        this._viewNotebook.set_current_page(this._GAME_VIEW_PAGE);
    },

    _switchToDashboardView: function() {
        this._viewNotebook.set_current_page(this._DASHBOARD_VIEW_PAGE);
    },

    _onPuzzleActivated: function(object, puzzleFile) {
        this._startGame(puzzleFile);
    }
});
